      subroutine calchmixplus(ps,ttlev,ulev,vlev,nuvz,
C                              i   i    i    i     i
     &   sigmahalf,ptop,p00,ts0,ta,hmix,hmixplus)
C            i      i    i   i   i  i    i      o
************************************************************************
* $Id: calchmixplus.f,v 1.1 2006/08/09 14:16:16 paul Exp paul $
* $Log: calchmixplus.f,v $
* Revision 1.1  2006/08/09 14:16:16  paul
* Initial revision
*
*                                                                      *
*                FLEXPART (MM5) SUBROUTINE CALCHMIXPLUS                *
*                          Author: G. WOTAWA                           *
*                              1998-08-10                              *
*                                                                      *
************************************************************************
*                                                                      *
* SHORT DESCRIPTION:                                                   *
*                                                                      *
* This subroutine calculates the excess mixing height needed for the   *
* subgridscale terrain variability  parameterisation from the          *
* brunt-vaissala frequency at z=hmix. hmix is a needed input.          *
*                                                                      *
************************************************************************
*                                                                      *
************************************************************************
*                                                                      *
* DESCRIPTION OF INPUT:                                                *
*                                                                      *
* PS              (REFERENCE) SURFACE PRESSURE (PA)                    *
* TTLEV           TEMPERATURE PROFILE ON HALF SIGMA LEVELS (K)         *
* ULEV            U WIND SPEED PROFILE ON HALF SIGMA LEVELS (M/S)      *
* VLEV            V WIND SPEED PROFILE ON HALF SIGMA LEVELS (M/S)      *
* TKELEV          TKE PROFILE ON HALF SIGMA LEVELS (M2/S2)             *
* NUVZ            NUMBER OF HALF SIGMA LEVELS                          *
* SIGMAHALF       HALF SIGMA LEVELS (DIMENSIONLESS)                    *
* PTOP            MM5 TOP PRESSURE (PA)                                *
* P00             REFERENCE SEA LEVEL PRESSURE (PA)                    *
* TS0             REFEENCE SEA LEVEL TEMPERATURE (K)                   *
* TA              REFERENCE TEMPERATURE LAPSE RATE (K)                 *
* HMIX            MIXING HEIGHT (M)                                    *
*                                                                      *
* DESCRIPTION OF OUTPUT:                                               *
*                                                                      *
* HMIXPLUS        EXCESS MIXING HEIGHT (M)                             *
*                                                                      *
************************************************************************
      include 'includepar'

      integer nuvz,k
      real ps,ttlev(nuvz),ulev(nuvz),vlev(nuvz),geopot1
      real sigmahalf(nuvz),ptop,p00,ts0,ta,zmin
      real hmix,hmixplus,z1,z2,rlin,sigmatoh,zz
      real ul,vl,p1,p2,theta1,theta2,thetam,bvfsq,bvf,wspeed

      geopot1=sigmatoh(sigmahalf(1),ps,ptop,p00,ts0,ta)  ! Geopotential of first mod. lev.

      zmin=sigmatoh(sigmahalf(2),ps,ptop,p00,ts0,ta)-geopot1
      if(hmix.lt.zmin) hmix=zmin  ! If mixing height is below height of
                                  ! first model half level, set it to first

      do 10 k=2,nuvz
        zz=sigmatoh(sigmahalf(k),ps,ptop,p00,ts0,ta)-geopot1
10      if(zz.ge.hmix) goto 20
20    continue

* Mixing height is situated between levels k-1,k
* -> interpolate all linearly

      z1=sigmatoh(sigmahalf(k-1),ps,ptop,p00,ts0,ta)-geopot1
      z2=sigmatoh(sigmahalf(k)  ,ps,ptop,p00,ts0,ta)-geopot1
      rlin=(hmix-z1)/(z2-z1)

* Compute hmixplus

      ul=ulev(k-1)+(ulev(k)-ulev(k-1))*rlin
      vl=vlev(k-1)+(vlev(k)-vlev(k-1))*rlin
      wspeed=sqrt(ul**2+vl**2)
      p1=sigmahalf(k-1)*(ps-ptop)+ptop
      p2=sigmahalf(k)  *(ps-ptop)+ptop
      theta1=ttlev(k-1)*(100000./p1)**kappa
      theta2=ttlev(k)  *(100000./p2)**kappa
      thetam=0.5*(theta1+theta2)
      bvfsq=(ga/thetam)*(theta2-theta1)/(z2-z1)
      if(bvfsq.le.0.) then
        hmixplus=4000.             ! Rising not confined by stability
      else
        bvf=sqrt(bvfsq)
        hmixplus=wspeed/bvf*convke ! Rising confined by stability
      endif

      return
      end
