      real function obukhov(ps,tlev1,qvlev1,ustar,hf)
*********************************************************************
* $Id: obukhov.f,v 1.1 2006/08/09 12:46:05 paul Exp paul $
* $Log: obukhov.f,v $
* Revision 1.1  2006/08/09 12:46:05  paul
* Initial revision
*
*                                                                   *
*                       Author: G. WOTAWA                           *
*                       Date:   1994-06-27                          *
*                                                                   *
*  Update: A. Stohl, 2000-09-25, avoid division by zero by          *
*  setting ustar to minimum value                                   *
*                                                                   *
*********************************************************************
*                                                                   *
*     This program calculates Obukhov scale height from surface     *
*     meteorological data and sensible heat flux.                   *
*                                                                   *
*********************************************************************
*                                                                   *
*     INPUT:                                                        *
*                                                                   *
*     ps      reference surface pressure [Pa]                       *
*     tlev1   temperature 1st model level [K]                       *
*     qvlev1  mixing ratio 1st level                                *
*     ustar   scale velocity [m/s]                                  *
*     hf      surface sensible heat flux [W/m2]                     *
*                                                                   *
*********************************************************************
 
      include 'includepar'
      real ps,tlev1,qvlev1,ustar,hf,tv,rhoa
      real theta,thetastar

      tv=tlev1*(1.+.378*qvlev1/(qvlev1+.622))
      rhoa=ps/(r_air*tv)
  
      theta=tlev1*(100000./ps)**kappa ! potential temperature
      if (ustar.le.0.) ustar=1.e-8
      thetastar=hf/(rhoa*cpa*ustar) ! scale temperature
      if(abs(thetastar).gt.1.e-10) then
         obukhov=theta*ustar**2/(karman*ga*thetastar)
      else
         obukhov=9999.           ! zero heat flux
      endif
      if (obukhov.gt. 9999.) obukhov= 9999.
      if (obukhov.lt.-9999.) obukhov=-9999.
	return

      end
