      subroutine outgrid_init()
********************************************************************************
* $Id: outgrid_init.f,v 1.1 2006/06/22 13:53:17 paul Exp paul $
* $Log: outgrid_init.f,v $
* Revision 1.1  2006/06/22 13:53:17  paul
* Initial revision
*
*                                                                              *
*  This routine calculates, for each grid cell of the output grid, the         *
*  volume, the surface area, and the areas of the northward and eastward       *
*  facing surfaces.                                                            *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     7 August 2002                                                            *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
*                                                                              *
* area               surface area of all output grid cells                     *
* areaeast           eastward facing wall area of all output grid cells        *
* areanorth          northward facing wall area of all output grid cells       *
* volume             volumes of all output grid cells                          *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      
      integer ix,jy,kz,k,i,nage,l,iix,jjy,ixp,jyp,i1,j1,j,ngrid
      real ylat,gridarea,ylatp,ylatm,hzone,cosfact,cosfactm,cosfactp
      real xlon,xl,yl,ddx,ddy,rddx,rddy,p1,p2,p3,p4,xtn,ytn,oroh
      real it,jt,dytmp,dxtmp,yaux,xaux,yaux1,xaux1,min,max


C Compute surface area and volume of each grid cell: area, volume;
C and the areas of the northward and eastward facing walls: areaeast, areanorth
************************************************************************

      do 10 jy=1,numygrid
      if(.not.lc_or_lamphi) then
        ylat=outlat0+(float(jy)+0.5)*dyout
        ylatp=ylat+0.5*dyout
        ylatm=ylat-0.5*dyout
        if ((ylatm.lt.0).and.(ylatp.gt.0.)) then
          hzone=dyout*r_earth*pi180
        else

C Calculate area of grid cell with formula M=2*pi*R*h*dx/360,
C see Netz, Formeln der Mathematik, 5. Auflage (1983), p.90
*************************************************************

          cosfact=cos(ylat*pi180)*r_earth
          cosfactp=cos(ylatp*pi180)*r_earth
          cosfactm=cos(ylatm*pi180)*r_earth
          if (cosfactp.lt.cosfactm) then
            hzone=sqrt(r_earth**2-cosfactp**2)-
     +      sqrt(r_earth**2-cosfactm**2)
          else
            hzone=sqrt(r_earth**2-cosfactm**2)-
     +      sqrt(r_earth**2-cosfactp**2)
          endif
        endif

C Surface are of a grid cell at a latitude ylat
***********************************************

        gridarea=2.*pi*r_earth*hzone*dxout/360.
      else
CPSK FIXXME
C Mapfactor should be applied here !?!
        gridarea=dxout*dyout
      endif
CPSK FIXXME
        do 10 ix=0,numxgrid-1
          area(ix,jy)=gridarea

C Volume = area x box height
****************************

          volume(ix,jy,1)=area(ix,jy)*outheight(1)
        if(.not.lc_or_lamphi) then
          areaeast(ix,jy,1)=dyout*r_earth*pi180*outheight(1)
          areanorth(ix,jy,1)=cos(ylat*pi180)*dxout*r_earth*pi180*
     +    outheight(1)
       else
          areaeast(ix,jy,1)=dyout*outheight(1)
          areanorth(ix,jy,1)=dxout*outheight(1)
       endif
       do 10 kz=2,numzgrid
          if(.not.lc_or_lamphi) then
             areaeast(ix,jy,kz)=dyout*r_earth*pi180*
     +            (outheight(kz)-outheight(kz-1))
             areanorth(ix,jy,kz)=cos(ylat*pi180)*dxout*r_earth*pi180*
     +            (outheight(kz)-outheight(kz-1))
          else
             areaeast(ix,jy,kz)=dyout*(outheight(kz)-outheight(kz-1))
             areanorth(ix,jy,kz)=dxout*(outheight(kz)-outheight(kz-1))
          endif
 10       volume(ix,jy,kz)=area(ix,jy)*(outheight(kz)-outheight(kz-1))



*******************************************************************
C Determine average height of model topography in output grid cells
*******************************************************************

C Loop over all output grid cells
*********************************
          min=9999.
          max=-9999.
      do 20 jjy=0,numygrid-1
        do 20 iix=0,numxgrid-1
          oroh=0.
          
          if(lc_or_lamphi) then
             if (iix.lt.numxgrid-1.and.jjy.lt.numygrid-1)then
                dytmp=outlat(iix,jjy+1)-outlat(iix,jjy)
                dxtmp=outlon(iix+1,jjy)-outlon(iix,jjy)
             endif
          endif

C Take 100 samples of the topography in every grid cell
*******************************************************

          do 22 j1=1,10
             do 22 i1=1,10
                if(.not.lc_or_lamphi) then
                   xlon=outlon0+(float(iix)+float(i1)/10.-0.05)*dxout
                   ylat=outlat0+(float(jjy)+float(j1)/10.-0.05)*dyout
                   call map_set(code_sav,lat1_sav,lon1_sav,1.,1.,
     &                  dx_sav,stdlon_sav,truelat1_sav,truelat2_sav)
                   call latlon_to_ij(ylat,xlon,xl,yl)
                else
                   xlon=outlon0+(float(iix)+float(i1)/10.-0.05)*dxtmp
                   ylat=outlat0+(float(jjy)+float(j1)/10.-0.05)*dytmp
!                   xl=(xlon-xlon0)/dx_sav
!                   yl=(ylat-ylat0)/dx_sav
                   call latlon_to_ij(ylat,xlon,xl,yl)
                endif
                if(xl.ge.max)max=xl
                if(xl.lt.min)min=xl

C Determine the nest we are in
******************************

              ngrid=0
              do 42 j=numbnests,1,-1
                if ((xl.gt.xln(j)).and.(xl.lt.xrn(j)).and.
     +          (yl.gt.yln(j)).and.(yl.lt.yrn(j))) then
                  ngrid=j
                  goto 43
                endif
42              continue
43            continue

C Determine (nested) grid coordinates and auxiliary parameters used for interpolation
*************************************************************************************

              if (ngrid.gt.0) then
!!                call ij_to_latlon(sngl(xl),sngl(yl),yaux,xaux)
!!                call ij_to_latlon(xln(ngrid),yln(ngrid),yaux1,xaux1)
!!                yaux=yaux-yaux1
!!                xaux=xaux-xaux1
!!                call map_set(code_sav,lat1n_sav(ngrid),lon1n_sav(ngrid),
!!     &ki_sav(ngrid),kj_sav(ngrid),dxn_sav(ngrid),stdlon_sav,truelat1_sav
!!     &,truelat2_sav) 
!!                call ij_to_latlon(1.,1.,yaux1,xaux1) 
!!                yaux=yaux+yaux1
!!                xaux=xaux+xaux1                
!!                call latlon_to_ij(yaux,xaux,xtn,ytn)
!!
!!!     Set map back to mother
!!                call map_set(code_sav,lat1_sav,lon1_sav,1.,1.,dx_sav,
!!     &               stdlon_sav,truelat1_sav,truelat2_sav)
!!
                xtn=(xl-xln(ngrid))*xresoln(ngrid)
                ytn=(yl-yln(ngrid))*yresoln(ngrid)
!                print*,xresoln,"?????"
!                print*,xtn-1.,(xl-xln(ngrid))*xresoln(ngrid)
!                print*,ytn-1.,(yl-yln(ngrid))*yresoln(ngrid)
!                pause
                ix=int(xtn)
                jy=int(ytn)
                ddy=ytn-float(jy)
                ddx=xtn-float(ix)
              else
                ix=int(xl)
                jy=int(yl)
                ddy=yl-float(jy)
                ddx=xl-float(ix)
              endif
              ixp=ix+1
              jyp=jy+1
              rddx=1.-ddx
              rddy=1.-ddy
              p1=rddx*rddy
              p2=ddx*rddy
              p3=rddx*ddy
              p4=ddx*ddy

              if (ngrid.gt.0) then
                oroh=oroh+p1*oron(ix ,jy ,ngrid)
     +                  + p2*oron(ixp,jy ,ngrid)
     +                  + p3*oron(ix ,jyp,ngrid)
     +                  + p4*oron(ixp,jyp,ngrid)
              else
                oroh=oroh+p1*oro(ix ,jy)
     +                  + p2*oro(ixp,jy)
     +                  + p3*oro(ix ,jyp)
     +                  + p4*oro(ixp,jyp)
              endif
22            continue

C Divide by the number of samples taken
***************************************

          oroout(iix,jjy)=oroh/100.
20        continue
 
*************************
C Initialize output grids
*************************

      do 31 k=1,nspec
        do 33 i=1,numreceptor
C Receptor points
33        creceptor(i,k)=0.
        do 31 nage=1,nageclass
          do 31 jy=0,numygrid-1
            do 31 ix=0,numxgrid-1
              do 31 l=1,nclassunc
C Deposition fields
                wetgridunc(ix,jy,k,l,nage)=0.
                drygridunc(ix,jy,k,l,nage)=0.
                do 31 kz=1,numzgrid
C Flux fields
                  fluxe(ix,jy,kz,k,nage)=0.
                  fluxw(ix,jy,kz,k,nage)=0.
                  fluxn(ix,jy,kz,k,nage)=0.
                  fluxs(ix,jy,kz,k,nage)=0.
                  fluxu(ix,jy,kz,k,nage)=0.
                  fluxd(ix,jy,kz,k,nage)=0.
C Concentration fields
31                gridunc(ix,jy,kz,k,l,nage)=0.

 

      end
