      subroutine outgrid_init_nest()
********************************************************************************
* $Id: outgrid_init_nest.f,v 1.1 2006/08/09 07:12:52 paul Exp paul $
* $Log: outgrid_init_nest.f,v $
* Revision 1.1  2006/08/09 07:12:52  paul
* Initial revision
*
*                                                                              *
*  This routine calculates, for each grid cell of the output nest, the         *
*  volume and the surface area.                                                *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*    30 August 2004                                                            *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
*                                                                              *
* arean              surface area of all output nest cells                     *
* volumen            volumes of all output nest cells                          *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      
      integer ix,jy,kz,k,nage,l,iix,jjy,ixp,jyp,i1,j1,j,ngrid
      real ylat,gridarea,ylatp,ylatm,hzone,cosfact,cosfactm,cosfactp
      real xlon,xl,yl,ddx,ddy,rddx,rddy,p1,p2,p3,p4,xtn,ytn,oroh
      real dytmp,dxtmp,yaux,xaux,yaux1,xaux1


C Compute surface area and volume of each grid cell: area, volume;
C and the areas of the northward and eastward facing walls: areaeast, areanorth
************************************************************************
 
       do 10 jy=0,numygridn-1
         if (.not. lc_or_lamphi) then
            ylat=outlat0n+(float(jy)+0.5)*dyoutn
            ylatp=ylat+0.5*dyoutn
            ylatm=ylat-0.5*dyoutn
            if ((ylatm.lt.0).and.(ylatp.gt.0.)) then
               hzone=dyoutn*r_earth*pi180
            else

C Calculate area of grid cell with formula M=2*pi*R*h*dx/360,
C see Netz, Formeln der Mathematik, 5. Auflage (1983), p.90
*************************************************************

          cosfact=cos(ylat*pi180)*r_earth
          cosfactp=cos(ylatp*pi180)*r_earth
          cosfactm=cos(ylatm*pi180)*r_earth
          if (cosfactp.lt.cosfactm) then
            hzone=sqrt(r_earth**2-cosfactp**2)-
     +      sqrt(r_earth**2-cosfactm**2)
          else
            hzone=sqrt(r_earth**2-cosfactm**2)-
     +      sqrt(r_earth**2-cosfactp**2)
          endif
        endif

C Surface are of a grid cell at a latitude ylat
***********************************************

        gridarea=2.*pi*r_earth*hzone*dxoutn/360.
      else
CPSK FIXXME
C Mapfactor should be applied here !?!
        gridarea=dxoutn*dyoutn
CPSK FIXXME
      endif
        do 10 ix=0,numxgridn-1
          arean(ix,jy)=gridarea

C Volume = area x box height
****************************

          volumen(ix,jy,1)=arean(ix,jy)*outheight(1)
          do 10 kz=2,numzgrid
10        volumen(ix,jy,kz)=arean(ix,jy)*(outheight(kz)-outheight(kz-1))


***************************************************************************
C Determine average height of model topography in nesteed output grid cells
***************************************************************************

C Loop over all output grid cells
*********************************

      do 20 jjy=0,numygridn-1
        do 20 iix=0,numxgridn-1
          oroh=0.

          if (lc_or_lamphi) then
             if (iix.lt.numxgridn-1.and.jjy.lt.numygridn-1)then
                dytmp=outlatn(iix,jjy+1)-outlatn(iix,jjy)
                dxtmp=outlonn(iix+1,jjy)-outlonn(iix,jjy)
             endif
          endif

C Take 100 samples of the topography in every grid cell
*******************************************************

          do 22 j1=1,10
             do 22 i1=1,10
                if (.not.lc_or_lamphi) then
                   xlon=outlon0n+(float(iix)+float(i1)/10.-0.05)*dxoutn
                   ylat=outlat0n+(float(jjy)+float(j1)/10.-0.05)*dyoutn
                   call map_set(code_sav,lat1_sav,lon1_sav,1.,1.,
     &                  dx_sav,stdlon_sav,truelat1_sav,truelat2_sav)
                   call latlon_to_ij(ylat,xlon,xl,yl)
C     xl=(xlon-xlon0)/dx
C     yl=(ylat-ylat0)/dy
                else
                   xlon=outlon0n+(float(iix)+float(i1)/10.-0.05)*dxtmp
                   ylat=outlat0n+(float(jjy)+float(j1)/10.-0.05)*dytmp
                   call latlon_to_ij(ylat,xlon,xl,yl)
                endif

C Determine the nest we are in
******************************

              ngrid=0
              do 42 j=numbnests,1,-1
                if ((xl.gt.xln(j)).and.(xl.lt.xrn(j)).and.
     +          (yl.gt.yln(j)).and.(yl.lt.yrn(j))) then
                  ngrid=j
                  goto 43
                endif
42              continue
43            continue

C Determine (nested) grid coordinates and auxiliary parameters used for interpolation
*************************************************************************************

              if (ngrid.gt.0) then
!                call ij_to_latlon(xl,yl,yaux,xaux)
!                call ij_to_latlon(xln(ngrid),yln(ngrid),yaux1,xaux1)
!                yaux=yaux-yaux1
!                xaux=xaux-xaux1
!                call map_set(code_sav,lat1n_sav(ngrid),lon1n_sav(ngrid),
!     &ki_sav(ngrid),kj_sav(ngrid),dxn_sav(ngrid),stdlon_sav,truelat1_sav
!     &,truelat2_sav) 
!                call ij_to_latlon(1.,1.,yaux1,xaux1) 
!                yaux=yaux+yaux1
!                xaux=xaux+xaux1                
!                call latlon_to_ij(yaux,xaux,xtn,ytn)

!     Set map back to mother
!                call map_set(code_sav,lat1_sav,lon1_sav,1.,1.,dx_sav,
!     &               stdlon_sav,truelat1_sav,truelat2_sav)
                xtn=(xl-xln(ngrid))*xresoln(ngrid)
                ytn=(yl-yln(ngrid))*yresoln(ngrid)
                ix=int(xtn)
                jy=int(ytn)
                ddy=ytn-float(jy)
                ddx=xtn-float(ix)
              else
                ix=int(xl)
                jy=int(yl)
                ddy=yl-float(jy)
                ddx=xl-float(ix)
              endif
              ixp=ix+1
              jyp=jy+1
              rddx=1.-ddx
              rddy=1.-ddy
              p1=rddx*rddy
              p2=ddx*rddy
              p3=rddx*ddy
              p4=ddx*ddy

              if (ngrid.gt.0) then
                oroh=oroh+p1*oron(ix ,jy ,ngrid)
     +                  + p2*oron(ixp,jy ,ngrid)
     +                  + p3*oron(ix ,jyp,ngrid)
     +                  + p4*oron(ixp,jyp,ngrid)
              else
                oroh=oroh+p1*oro(ix ,jy)
     +                  + p2*oro(ixp,jy)
     +                  + p3*oro(ix ,jyp)
     +                  + p4*oro(ixp,jyp)
              endif
22            continue

C Divide by the number of samples taken
***************************************

          orooutn(iix,jjy)=oroh/100.
20        continue



********************************
C Initialization of output grids
********************************

      do 31 k=1,nspec
        do 31 nage=1,nageclass
          do 31 jy=0,numygridn-1
            do 31 ix=0,numxgridn-1
              do 31 l=1,nclassunc
C Deposition fields
                wetgriduncn(ix,jy,k,l,nage)=0.
                drygriduncn(ix,jy,k,l,nage)=0.
C Concentration fields
                do 31 kz=1,numzgrid
31                griduncn(ix,jy,kz,k,l,nage)=0.

 

      end
