      subroutine verttransform_nests(n,uuhn,vvhn,wwhn)
C                                    i   i    i    i   i
********************************************************************************
* $Id: verttransform_nests.f,v 1.1 2006/08/11 14:45:26 paul Exp paul $
* $Log: verttransform_nests.f,v $
* Revision 1.1  2006/08/11 14:45:26  paul
* Initial revision
* Revision 1.2  20080705 P. Seibert
* - Replace grid point (1,1) by the lowest one for level construction
*   as in other newer Flexpart versions
* - Get rid of doubling of layers; 
* - correct the w correction scheme which was wrong in all Flexpart-MM5 
*   versions up to now.
*
*                                                                              *
*     This subroutine transforms temperature, dew point temperature and        *
*     wind components from eta to meter coordinates.                           *
*     The vertical wind component is transformed from Pa/s to m/s using        *
*     the conversion factor pinmconv.                                          *
*     In addition, this routine calculates vertical density gradients          *
*     needed for the parameterization of the turbulent velocities.             *
*     It is similar to verttransform, but makes the transformations for        *
*     the nested grids.                                                        *
*                                                                              *
*     Author: A. Stohl, G. Wotawa                                              *
*                                                                              *
*     12 August 1996                                                           *
*     Update: 16 January 1998                                                  *
*                                                                              *
*     Major update: 17 February 1999                                           *
*     by G. Wotawa                                                             *
*                                                                              *
*     - Vertical levels for u, v and w are put together                        *
*     - Slope correction for vertical velocity: Modification of calculation    *
*       procedure                                                              *
*                                                                              *
********************************************************************************
*  Changes, Bernd C. Krueger, Feb. 2001:       (marked "C-cv")
*        Variables tthn and qvhn (on eta coordinates) from common block
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* nxn,nyn,nuvz,nwz                field dimensions in x,y and z direction      *
* uun                             wind components in x-direction [m/s]         *
* vvn                             wind components in y-direction [m/s]         *
* wwn                             wind components in z-direction [deltaeta/s]  *
* ttn                             temperature [K]                              *
* pvn                             potential vorticity (pvu)                    *
* psn                             surface pressure [Pa]                        *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer ix,jy,kz,iz,n,l,kmin,kl,klp,ix1,jy1,ixp,jyp
      real uvzlev(nuvzmax),wzlev(nwzmax),rhoh(nuvzmax),pinmconv(nzmax)
      real uvwzlev(0:nxmaxn-1,0:nymaxn-1,nzmax)
      real ew,pint,tv,tvold,pold,const,dz1,dz2,dz,ui,vi
      real dzdx,dzdy
      real dzdx1,dzdx2,dzdy1,dzdy2
      real uuhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
      real vvhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
!      real pvhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
      real wwhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests) !nwzmax
      parameter(const=r_air/ga)
      real geopot1,sigmatoh
!      real tkehn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
      real wm,dxtmp,dytmp

C Loop over all nests
*********************

      do 100 l=1,numbnests

C Loop over the whole grid
**************************

      do 10 jy=0,nyn(l)-1
        do 10 ix=0,nxn(l)-1

           geopot1=sigmatoh(sigmafull(1),psn(ix,jy,1,n,l),ptop,p00,
     &          ts0,ta)
           
C          tvold=tt2n(ix,jy,1,n,l)*(1.+0.378*ew(td2n(ix,jy,1,n,l))/
C     &                                   psn(ix,jy,1,n,l))
C          pold=psn(ix,jy,1,n,l)
          uvzlev(1)=0.
          wzlev(1)=0.

          wm=qvhn(ix,jy,1,n,l)
          tv=tthn(ix,jy,1,n,l)*(1.+0.378*wm/(wm+.622))
          rhoh(1)=psn(ix,jy,1,n,l)/(r_air*tv)
C          rhoh(1)=pold/(r_air*tvold)


C Compute heights of eta levels
*******************************

          do 20 kz=2,nuvz
            pint=sigmahalf(kz)*(psn(ix,jy,1,n,l)-ptop)+ptop
            wm=qvhn(ix,jy,kz,n,l)/(1.-qvhn(ix,jy,kz,n,l))
            tv=tthn(ix,jy,kz,n,l)*(1.+0.378*wm/(wm+.622))
            rhoh(kz)=pint/(r_air*tv)

            uvzlev(kz)=sigmatoh(sigmahalf(kz),psn(ix,jy,1,n,l),ptop,
     &        p00,ts0,ta)-geopot1

C            tv=tthn(ix,jy,kz,n,l)*(1.+0.608*qvhn(ix,jy,kz,n,l))
C            rhoh(kz)=pint/(r_air*tv)

C            if (abs(tv-tvold).gt.0.2) then
C              uvzlev(kz)=uvzlev(kz-1)+const*log(pold/pint)*
C     +        (tv-tvold)/log(tv/tvold)
C            else
C              uvzlev(kz)=uvzlev(kz-1)+const*log(pold/pint)*tv
C            endif
           
C            tvold=tv
C20          pold=pint
 20      continue

          do 21 kz=2,nwz-1
            wzlev(kz)=sigmatoh(sigmafull(kz),psn(ix,jy,1,n,l),ptop,
     &        p00,ts0,ta)-geopot1
 21      continue
 
          uvwzlev(ix,jy,1)=0.0
          do 22 kz=2,nuvz
 22          uvwzlev(ix,jy,kz)=uvzlev(kz)


C Levels where u,v,t and q are given
*************************************

          uun(ix,jy,1,n,l)=uuhn(ix,jy,1,l)
          vvn(ix,jy,1,n,l)=vvhn(ix,jy,1,l)
          ttn(ix,jy,1,n,l)=tthn(ix,jy,1,n,l)
          qvn(ix,jy,1,n,l)=qvhn(ix,jy,1,n,l)
!          pvn(ix,jy,1,n,l)=pvhn(ix,jy,1,l)
          tken(ix,jy,1,n,l)=tkehn(ix,jy,1,l)
          rhon(ix,jy,1,n,l)=rhoh(1)

          uun(ix,jy,nz,n,l)=uuhn(ix,jy,nuvz,l)
          vvn(ix,jy,nz,n,l)=vvhn(ix,jy,nuvz,l)
          ttn(ix,jy,nz,n,l)=tthn(ix,jy,nuvz,n,l)
          qvn(ix,jy,nz,n,l)=qvhn(ix,jy,nuvz,n,l)
!          pvn(ix,jy,nz,n,l)=pvhn(ix,jy,nuvz,l)
          tken(ix,jy,nz,n,l)=tkehn(ix,jy,nuvz,l)
          rhon(ix,jy,nz,n,l)=rhoh(nuvz)

          kmin=2
          do 30 iz=2,nz-1
            do 35 kz=kmin,nuvz
              if(height(iz).gt.uvzlev(nuvz)) then
                uun(ix,jy,iz,n,l)=uun(ix,jy,nz,n,l)
                vvn(ix,jy,iz,n,l)=vvn(ix,jy,nz,n,l)
                ttn(ix,jy,iz,n,l)=ttn(ix,jy,nz,n,l)
                qvn(ix,jy,iz,n,l)=qvn(ix,jy,nz,n,l)
!                pvn(ix,jy,iz,n,l)=pvn(ix,jy,nz,n,l)
                tken(ix,jy,iz,n,l)=tkehn(ix,jy,1,l)                
                rhon(ix,jy,iz,n,l)=rhon(ix,jy,nz,n,l)
                goto 30
              endif
              if ((height(iz).gt.uvzlev(kz-1)).and.
     +            (height(iz).le.uvzlev(kz))) then
               dz1=height(iz)-uvzlev(kz-1)
               dz2=uvzlev(kz)-height(iz)
               dz=dz1+dz2
               uun(ix,jy,iz,n,l)=(uuhn(ix,jy,kz-1,l)*dz2+
     +         uuhn(ix,jy,kz,l)*dz1)/dz
               vvn(ix,jy,iz,n,l)=(vvhn(ix,jy,kz-1,l)*dz2+
     +         vvhn(ix,jy,kz,l)*dz1)/dz
               ttn(ix,jy,iz,n,l)=(tthn(ix,jy,kz-1,n,l)*dz2+
     +         tthn(ix,jy,kz,n,l)*dz1)/dz
               qvn(ix,jy,iz,n,l)=(qvhn(ix,jy,kz-1,n,l)*dz2+
     +         qvhn(ix,jy,kz,n,l)*dz1)/dz
!               pvn(ix,jy,iz,n,l)=(pvhn(ix,jy,kz-1,l)*dz2+
!     +         pvhn(ix,jy,kz,l)*dz1)/dz
               tken(ix,jy,iz,n,l)=(tkehn(ix,jy,kz-1,l)*dz2+
     +              tkehn(ix,jy,kz,l)*dz1)/dz              
               rhon(ix,jy,iz,n,l)=(rhoh(kz-1)*dz2+rhoh(kz)*dz1)/dz
               kmin=kz
               goto 30
              endif
35            continue
30          continue


C Levels where w is given
**************************

          wwn(ix,jy,1,n,l)=wwhn(ix,jy,1,l) 
          wwn(ix,jy,nz,n,l)=wwhn(ix,jy,nwz,l) 
          kmin=2
          do 40 iz=2,nz
            do 45 kz=kmin,nwz
              if ((height(iz).gt.wzlev(kz-1)).and.
     +            (height(iz).le.wzlev(kz))) then
               dz1=height(iz)-wzlev(kz-1)
               dz2=wzlev(kz)-height(iz)
               dz=dz1+dz2
               wwn(ix,jy,iz,n,l)=(wwhn(ix,jy,kz-1,l)*dz2+
     +         wwhn(ix,jy,kz,l)*dz1)/dz !*pinmconv(iz)
               kmin=kz
               goto 40
              endif
45            continue
40          continue

C Compute density gradients at intermediate levels
**************************************************

          drhodzn(ix,jy,1,n,l)=(rhon(ix,jy,2,n,l)-rhon(ix,jy,1,n,l))/
     +      (height(2)-height(1))
          do 50 kz=2,nz-1
50          drhodzn(ix,jy,kz,n,l)=(rhon(ix,jy,kz+1,n,l)-
     +      rhon(ix,jy,kz-1,n,l))/(height(kz+1)-height(kz-1))
          drhodzn(ix,jy,nz,n,l)=drhodzn(ix,jy,nz-1,n,l)

10        continue


*****************************************************************
C Compute slope of Flexpart  levels in windward direction and resulting
C vertical wind correction
*****************************************************************

      do 60 jy=1,nyn(l)-2
        do 60 ix=1,nxn(l)-2

          ix1=ix-1
          jy1=jy-1
          ixp=ix+1
          jyp=jy+1

CPS getting the slope of the Flexpart z levels is trivial, as they
CPS are parallel to the model orography
          dzdx=(oron(ixp,jy,l)-oron(ix1,jy,l))/2.
          dzdy=(oron(ix,jyp,l)-oron(ix,jy1,l))/2.

          kmin=2
          do 41 iz=1,nz
	    
CPSK FIXXME / by griddistance in m dx=dy in MM5!!!
CPS dxconst=dyconst= 1./dx where dx is the grid distance in metres
CPS at true latitude. Should be replaced by exact distances.
            ui=uun(ix,jy,iz,n,l)*dxconst*xresoln(l) 
            vi=vvn(ix,jy,iz,n,l)*dyconst*yresoln(l)

CPS ww as it comes from MM5 is ordinary vertical velocity (individual
CPS change of height above sea level. In Flexpart we need vertical velocity
CPS in m/s but relative to model topography. So if a particle moves upslope
CPS parallel to the ground, it should have ww=0. Its velocity in MM5 will be
CPS dzdx*ui + dzdy*vi (positive if oro increases along the wind direction)
CPS Thus we have to subtract this term.

              wwn(ix,jy,iz,n,l)=wwn(ix,jy,iz,n,l) - (dzdx*ui + dzdy*vi)
 
41          continue
60         continue
100       continue

C Note that polar region treatment is not implemented

      end
